@@ -2,15 +2,17 @@
 switchboard.base
 ~~~~~~~~~~~~~~~
 
-:copyright: (c) 2012 SourceForge.
+:copyright: (c) 2015 Kyle Adams.
 :license: Apache License 2.0, see LICENSE for more details.
 """
 
 import time
 import logging
+import threading
 
 from .models import MongoModel
 from .signals import request_finished
+from .settings import settings
 
 log = logging.getLogger(__name__)
 
@@ -18,26 +20,35 @@ log = logging.getLogger(__name__)
 NoValue = object()
 
 
-class CachedDict(object):
-    def __init__(self, cache=None, timeout=30):
+class CachedDict(threading.local):
+    def __init__(self, timeout=30):
+        """
+        Not guaranteed to be called with expected c'tor args of
+        all usages (due to usage of threading.local)
+        """
         cls_name = type(self).__name__
 
         self._cache = None
         self._last_updated = None
         self.timeout = timeout
-        self.cache = cache
+        self.cache = settings.SWITCHBOARD_CACHE
         self.cache_key = cls_name
-        self.last_updated_cache_key = '%s.last_updated' % (cls_name,)
+        self.last_updated_cache_key = f'{cls_name}.last_updated'
 
     def __getitem__(self, key):
         self._populate()
         try:
-            return self._cache[key]
+            value = self._cache[key]
         except KeyError:
             value = self.get_default(key)
             if value is NoValue:
                 raise
-            return value
+        except (TypeError, AttributeError):
+            log.exception('Unable to access the local cache')
+            value = self.get_default(key)
+            if value is NoValue:
+                raise KeyError(key)
+        return value
 
     def __len__(self):  # pragma: nocover
         if self._cache is None:
@@ -57,25 +68,25 @@ class CachedDict(object):
 
     def iteritems(self):  # pragma: nocover
         self._populate()
-        return self._cache.iteritems()
+        return self._cache.items()
 
     def itervalues(self):  # pragma: nocover
         self._populate()
-        return self._cache.itervalues()
+        return self._cache.values()
 
     def iterkeys(self):  # pragma: nocover
         self._populate()
-        return self._cache.iterkeys()
+        return self._cache.keys()
 
     def keys(self):  # pragma: nocover
-        return list(self.iterkeys())
+        return list(self.keys())
 
     def values(self):  # pragma: nocover
-        return list(self.itervalues())
+        return list(self.values())
 
     def items(self):  # pragma: nocover
         self._populate()
-        return self._cache.items()
+        return list(self._cache.items())
 
     def get(self, key, default=None):
         self._populate()
@@ -129,7 +140,7 @@ class CachedDict(object):
         if not cache_last_updated:
             return None
 
-        if int(cache_last_updated) > self._last_updated:
+        if int(cache_last_updated) > (self._last_updated or 0):
             return True
 
         return False
@@ -260,7 +271,7 @@ class MongoModelDict(CachedDict):
                  auto_create=False, *args, **kwargs):
         assert value is not None
 
-        super(MongoModelDict, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         cls_name = type(self).__name__
         name = model.__name__
@@ -271,8 +282,8 @@ class MongoModelDict(CachedDict):
         self.model = model
         self.auto_create = auto_create
 
-        self.cache_key = '%s:%s:%s' % (cls_name, name, self.key)
-        self.last_updated_cache_key = '%s.last_updated:%s:%s' % (cls_name,
+        self.cache_key = f'{cls_name}:{name}:{self.key}'
+        self.last_updated_cache_key = '{}.last_updated:{}:{}'.format(cls_name,
                                                                  name,
                                                                  self.key)
         request_finished.connect(self._cleanup)
@@ -311,7 +322,7 @@ class MongoModelDict(CachedDict):
         return result
 
     def _get_cache_data(self):
-        return dict((getattr(i, self.key), i) for i in self.model.all())
+        return {getattr(i, self.key): i for i in self.model.all()}
 
     # Signals
     def _post_save(self, sender, **kwargs):
