@@ -2,7 +2,7 @@
 switchboard.helpers
 ~~~~~~~~~~~~~~~~
 
-:copyright: (c) 2012 SourceForge.
+:copyright: (c) 2015 Kyle Adams.
 :license: Apache License 2.0, see LICENSE for more details.
 """
 
@@ -10,24 +10,10 @@ import logging
 from collections import defaultdict
 from copy import deepcopy
 
-from webob import Request
-
 log = logging.getLogger(__name__)
 
 
-class MockRequest(Request):
-    """
-    A mock request object which stores a user
-    instance and the ip address.
-    """
-    def __init__(self, user=None, ip_address=None):
-        blank = Request.blank('/')
-        blank.environ['REMOTE_ADDR'] = ip_address
-        super(MockRequest, self).__init__(blank.environ)
-        self.user = user
-
-
-class MockCollection(object):
+class MockCollection:
     """
     A quick and dirty implementation of PyMongo's Collection API,
     to allow for easy testing without a DB connection.
@@ -38,8 +24,8 @@ class MockCollection(object):
         self.database = defaultdict(lambda: MockCollection(), name=self)
 
     def _matches(self, spec, document):
-        for k, v in spec.iteritems():
-            if not document.get(k) or document[k] != v:
+        for k, v in spec.items():
+            if k not in document or document[k] != v:
                 return False
         return True
 
@@ -47,7 +33,7 @@ class MockCollection(object):
         result = self.find(spec)
         return result[0] if result else None
 
-    def find(self, spec=None):
+    def find(self, spec=None, sort=None):
         results = []
         if not spec:
             # Return a copy of the list so that updating the returned list does
@@ -59,19 +45,23 @@ class MockCollection(object):
         return results or None
 
     def _update_partial(self, old, new):
-        for k, v in new.iteritems():
+        for k, v in new.items():
             old[k] = v
 
-    def update(self, spec, update):
+    def update(self, spec, update, upsert=False):
         current = self.find_one(spec)
         if not current:
-            return {
-                'err': None,
-                'n': 1,
-                'ok': 1.0,
-                'updatedExisting': True
-            }
-        for k, v in update.iteritems():
+            if upsert:
+                spec.update(update)
+                return self.save(spec)
+            else:
+                return {
+                    'err': None,
+                    'n': 1,
+                    'ok': 1.0,
+                    'updatedExisting': True
+                }
+        for k, v in update.items():
             if k == '$set':
                 self._update_partial(current, v)
             else:
