@@ -2,16 +2,15 @@
 switchboard.manager
 ~~~~~~~~~~~~~~~~
 
-:copyright: (c) 2012 SourceForge.
+:copyright: (c) 2015 Kyle Adams.
 :license: Apache License 2.0, see LICENSE for more details.
 """
 
 import logging
-from decorator import decorator
 
-from pymongo import Connection
+import pymongo
+from pymongo.mongo_client import MongoClient
 
-from . import signals
 from .base import MongoModelDict
 from .models import (
     MongoModel,
@@ -23,52 +22,60 @@ from .proxy import SwitchProxy
 from .settings import settings, Settings
 
 log = logging.getLogger(__name__)
+# These are (mostly) read-only module variables since we want it shared among
+# any and all threads. The only exception to read-only is when they are
+# populated on Switchboard startup (i.e., operator.register()).
+registry = {}
+registry_by_namespace = {}
 
 
 def nested_config(config):
     cfg = {}
     token = 'switchboard.'
-    for k, v in config.iteritems():
+    for k, v in config.items():
         if k.startswith(token):
             cfg[k.replace(token, '')] = v
     return cfg
 
 
-def configure(config={}, nested=False):
+def configure(config={}, nested=False, cache=None, allow_no_mongo=False):
     """
     Useful for when you need to control Switchboard's setup
     """
     if nested:
         config = nested_config(config)
     # Re-read settings to make sure we have everything
-    Settings.init(**config)
+    Settings.init(cache=cache, **config)
+
+    operator.cache = cache
+
     # Establish the connection to Mongo
     mongo_timeout = getattr(settings, 'SWITCHBOARD_MONGO_TIMEOUT', None)
-    # The config is in ms to match memcached, but pymongo wants seconds
-    mongo_timeout = mongo_timeout // 1000 if mongo_timeout else mongo_timeout
     # Ensure we have an integer for port and not a string
     mongo_port = int(settings.SWITCHBOARD_MONGO_PORT)
     try:
-        conn = Connection(settings.SWITCHBOARD_MONGO_HOST,
-                          mongo_port,
-                          network_timeout=mongo_timeout)
+        more_settings = {}
+        if pymongo.version_tuple[0] >= 3:
+            more_settings['serverSelectionTimeoutMS'] = mongo_timeout
+        conn = MongoClient(settings.SWITCHBOARD_MONGO_HOST,
+                           mongo_port,
+                           socketTimeoutMS=mongo_timeout,
+                           connectTimeoutMS=mongo_timeout,
+                           **more_settings
+                           )
+        conn.admin.command('ping')  # make sure we're actually connected
         db = conn[settings.SWITCHBOARD_MONGO_DB]
         collection = db[settings.SWITCHBOARD_MONGO_COLLECTION]
         Switch.c = collection
-    except:
-        log.exception('Unable to connect to the datastore')
+    except Exception:
+        if allow_no_mongo:
+            log.exception('Unable to connect to the datastore, will use in-memory switch collection')
+        else:
+            raise
     # Register the builtins
     __import__('switchboard.builtins')
 
 
-@decorator
-def switch_checked(func, *args, **kwargs):
-    active = func(*args, **kwargs)
-    key = args[1]
-    signals.switch_checked.send(key, active=active)
-    return active
-
-
 class SwitchManager(MongoModelDict):
     DISABLED = DISABLED
     SELECTIVE = SELECTIVE
@@ -79,8 +86,6 @@ class SwitchManager(MongoModelDict):
     EXCLUDE = EXCLUDE
 
     def __init__(self, *args, **kwargs):
-        # We'll store available conditions in the registry
-        self._registry = {}
         # Inject args and kwargs that are known quantities; the SwitchManager
         # will always deal with the Switch model and so on.
         new_args = [Switch]
@@ -88,14 +93,16 @@ class SwitchManager(MongoModelDict):
             new_args.append(a)
         kwargs['key'] = 'key'
         kwargs['value'] = 'value'
+        self.result_cache = None
+        self.context = {}
         MongoModel.post_save.connect(self.version_switch)
         MongoModel.post_delete.connect(self.version_switch)
-        super(SwitchManager, self).__init__(*new_args, **kwargs)
+        super().__init__(*new_args, **kwargs)
 
-    def __unicode__(self):
-        return "<%s: %s (%s)>" % (self.__class__.__name__,
+    def __str__(self):
+        return "<{}: {} ({})>".format(self.__class__.__name__,
                                   getattr(self, 'model', ''),
-                                  self._registry.values())
+                                  list(registry.values()))
 
     def __getitem__(self, key):
         """
@@ -103,15 +110,40 @@ class SwitchManager(MongoModelDict):
         easily extend the Switches method and automatically include our
         manager instance.
         """
-        return SwitchProxy(self, super(SwitchManager, self).__getitem__(key))
+        return SwitchProxy(self, super().__getitem__(key))
 
-    @switch_checked
+    def with_result_cache(func):
+        """
+        Decorator specifically for is_active.  If self.result_cache is set to a {}
+        the is_active results will be cached for each set of params.
+        """
+        def inner(self, *args, **kwargs):
+            dic = self.result_cache
+            cache_key = None
+            if dic is not None:
+                cache_key = (args, tuple(sorted(kwargs.items())) if kwargs else ())
+                try:
+                    result = dic.get(cache_key)
+                except TypeError as e:  # not hashable
+                    log.debug('Switchboard result cache not active for this "%s" check due to: %s within args: %s',
+                              args[0], e, repr(cache_key)[:200])
+                    cache_key = None
+                else:
+                    if result is not None:
+                        return result
+            result = func(self, *args, **kwargs)
+            if cache_key is not None:
+                dic[cache_key] = result
+            return result
+        return inner
+
+    @with_result_cache
     def is_active(self, key, *instances, **kwargs):
         """
         Returns ``True`` if any of ``instances`` match an active switch.
         Otherwise returns ``False``.
 
-        >>> opersator.is_active('my_feature', request) #doctest: +SKIP
+        >>> operator.is_active('my_feature', request) #doctest: +SKIP
         """
         try:
             default = kwargs.pop('default', False)
@@ -133,7 +165,6 @@ class SwitchManager(MongoModelDict):
                 switch = self[key]
             except KeyError:
                 # switch is not defined, defer to parent
-                signals.switch_checked.send(None, active=default)
                 return default
 
             if switch.status == GLOBAL:
@@ -149,18 +180,16 @@ class SwitchManager(MongoModelDict):
                 return default
 
             instances = list(instances) if instances else []
-            request = self.get_request()
-            if request:
-                instances.append(request)
-            user = self.get_user()
-            if user:
-                instances.append(user)
+            instances.extend(list(self.context.values()))
 
             # check each switch to see if it can execute
             return_value = False
 
-            for condition_set in self._registry.itervalues():
-                result = condition_set.has_active_condition(conditions,
+            for namespace, condition in conditions.items():
+                condition_set = registry_by_namespace.get(namespace)
+                if not condition_set:
+                    continue
+                result = condition_set.has_active_condition(condition,
                                                             instances)
                 if result is False:
                     return False
@@ -183,7 +212,8 @@ class SwitchManager(MongoModelDict):
 
         if callable(condition_set):
             condition_set = condition_set()
-        self._registry[condition_set.get_id()] = condition_set
+        registry[condition_set.get_id()] = condition_set
+        registry_by_namespace[condition_set.get_namespace()] = condition_set
 
     def unregister(self, condition_set):
         """
@@ -193,21 +223,22 @@ class SwitchManager(MongoModelDict):
         """
         if callable(condition_set):
             condition_set = condition_set()
-        self._registry.pop(condition_set.get_id(), None)
+        registry.pop(condition_set.get_id(), None)
+        registry_by_namespace.pop(condition_set.get_namespace(), None)
 
     def get_condition_set_by_id(self, switch_id):
         """
         Given the identifier of a condition set (described in
         ConditionSet.get_id()), returns the registered instance.
         """
-        return self._registry[switch_id]
+        return registry[switch_id]
 
     def get_condition_sets(self):
         """
         Returns a generator yielding all currently registered
         ConditionSet instances.
         """
-        return self._registry.itervalues()
+        return registry.values()
 
     def get_all_conditions(self):
         """
@@ -218,32 +249,40 @@ class SwitchManager(MongoModelDict):
         """
         cs = self.get_condition_sets()
         for condition_set in sorted(cs, key=lambda x: x.get_group_label()):
-            group = unicode(condition_set.get_group_label())
-            for field in condition_set.fields.itervalues():
+            group = str(condition_set.get_group_label())
+            for field in condition_set.fields.values():
                 yield condition_set.get_id(), group, field
 
-    def as_request(self, user=None, ip_address=None):
-        from .helpers import MockRequest
-
-        return MockRequest(user, ip_address)
-
-    def get_request(self):
-        """
-        A callback to get the request object should be setup during
-        switchboard setup.
-        """
-        pass
-
-    def get_user(self):
-        """
-        A callback to get the request object should be setup during
-        switchboard setup.
-        """
-        pass
-
     def version_switch(self, switch):
-        user = self.get_user()
-        switch.save_version(username=user.username if user else '')
+        '''
+        Save changes made to a switch. Triggered by create and update events
+        on a switch model. The changes are saved as diffs and reassembled to
+        create a switch history. Allows changes to switches to be audited.
+        '''
+        # Try to get the username from both User objects and user dicts.
+        user = self.context.get('user', {})
+        username = ''
+
+        try:
+            username = user.username
+        except AttributeError:
+            # doesn't have that attr, ok
+            pass
+        except Exception:
+            # @property username threw some other error
+            log.debug('Error when trying user.username attr', exc_info=True)
+
+        if not username:
+            try:
+                username = user.get('username', '')
+            except AttributeError:
+                # not a dict, ok
+                pass
+
+        try:
+            switch.save_version(username=username)
+        except Exception:
+            log.warning('Unable to save the switch version', exc_info=True)
 
 
 auto_create = getattr(settings, 'SWITCHBOARD_AUTO_CREATE', True)
