@@ -48,7 +48,7 @@ class MockModel(VersioningMongoModel):
 
 class TestMongoModelDict:
 
-    def teardown(self):
+    def teardown_method(self):
         MockModel.c.drop()
 
     def test_api(self):
@@ -78,7 +78,7 @@ class TestMongoModelDict:
 
         mydict = MongoModelDict(MockModel, key='key', value='value')
 
-        assert mydict._cache == None
+        assert mydict._cache is None
 
         instance = MockModel(key='test_expirey', value='hello')
         mydict['test_expirey'] = instance
@@ -88,7 +88,7 @@ class TestMongoModelDict:
 
         request_finished.send(Mock())
 
-        assert mydict._last_updated == None
+        assert mydict._last_updated is None
         assert mydict['test_expirey'] == instance
         assert len(mydict._cache) == base_count + 1
 
@@ -154,14 +154,14 @@ class TestMongoModelDict:
 
 
 class TestCacheIntegration:
-    def setup(self):
+    def setup_method(self):
         self.cache = Mock()
         self.cache.get.return_value = {}
         self.mydict = MongoModelDict(MockModel, key='key', value='value',
                                      auto_create=True)
         self.mydict.cache = self.cache
 
-    def teardown(self):
+    def teardown_method(self):
         MockModel.c.drop()
 
     def test_model_creation(self):
@@ -246,7 +246,7 @@ class TestCacheIntegration:
 
 
 class TestCachedDict:
-    def setup(self):
+    def setup_method(self):
         self.cache = Mock()
         self.mydict = CachedDict(timeout=100)
         self.mydict.cache =self.cache
@@ -321,7 +321,7 @@ class TestCachedDict:
 
         last_updated = self.mydict.last_updated_cache_key
         self.cache.get.assert_called_once_with(last_updated)
-        assert result == False
+        assert result is False
 
     def test_is_expired_if_remote_cache_is_new(self):
         # set it to an expired time
@@ -332,7 +332,7 @@ class TestCachedDict:
 
         last_updated = self.mydict.last_updated_cache_key
         self.cache.get.assert_called_once_with(last_updated)
-        assert result == True
+        assert result is True
 
     def test_is_expired_if_never_updated(self):
         # _last_updated None
@@ -341,7 +341,7 @@ class TestCachedDict:
 
         result = self.mydict.has_global_changed()
 
-        assert result == True
+        assert result is True
 
     @patch('switchboard.base.CachedDict._populate')
     @patch('switchboard.base.CachedDict.get_default')
@@ -354,7 +354,7 @@ class TestCachedDict:
 
 class TestCacheConcurrency:
 
-    def setup(self):
+    def setup_method(self):
         self.mydict = CachedDict()
         self.exc = None
 
